/*
 * Copyright 2007 Future Earth, info@future-earth.eu
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package eu.future.earth.gwt.client.date.list;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;

import org.cobogw.gwt.user.client.CSS;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.logical.shared.ResizeEvent;
import com.google.gwt.event.logical.shared.ResizeHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ResizeComposite;
import com.google.gwt.user.client.ui.ScrollPanel;

import eu.future.earth.gwt.client.FtrGwtDateCss;
import eu.future.earth.gwt.client.date.DateEvent;
import eu.future.earth.gwt.client.date.DateEvent.DateEventActions;
import eu.future.earth.gwt.client.date.DateEventListener;
import eu.future.earth.gwt.client.date.DatePanel;
import eu.future.earth.gwt.client.date.DateRenderer;
import eu.future.earth.gwt.client.date.EventComparator;
import eu.future.earth.gwt.client.date.EventPanel;
import eu.future.earth.gwt.client.date.HasDateEventHandlers;
import eu.future.earth.gwt.client.date.PanelType;
import eu.future.earth.gwt.client.utils.General;

public class EventListPanel<T> extends ResizeComposite implements ResizeHandler, DateEventListener<T>, DatePanel<T>, HasDateEventHandlers<T>, ClickHandler {

	private List<EventPanel<T>> events = new ArrayList<EventPanel<T>>();

	private Grid listPanel = new Grid(2, 3);

	private FlowPanel top = new FlowPanel();

	private ScrollPanel scroller = new ScrollPanel(top);

	public int getFirstDayOfWeek() {
		return firstDayOfWeek;
	}

	private int firstDayOfWeek = Calendar.MONDAY;

	private DateRenderer<T> renderer = null;

	public EventListPanel(DateRenderer<T> newRenderer) {
		super();
		setRenderer(newRenderer);
		initWidget(scroller);
		top.add(listPanel);
		scroller.setStyleName(FtrGwtDateCss.WEEK_SCROLLER);
		CSS.setProperty(listPanel, CSS.A.WIDTH, "100%");
		CSS.setProperty(top, CSS.A.WIDTH, "100%");
		listPanel.setCellPadding(0);
		listPanel.setCellSpacing(0);

	}

	public final void setRenderer(DateRenderer<T> newRenderer) {
		renderer = newRenderer;
		repaintData();
	}

	public void setDate(Date newDate) {
		current.setTime(newDate);
		setLast();
		repaintData();
	}

	private void setLast() {
		final Calendar helperCal = new GregorianCalendar();
		helperCal.setTime(current.getTime());
		helperCal.add(Calendar.WEEK_OF_YEAR, 2);
		lastShow = helperCal.getTime();
	}

	private Date lastShow = null;

	@SuppressWarnings("unchecked")
	public void addEventData(T newEvent) {
		EventPanel<T> newPanel = (EventPanel<T>) renderer.createPanel(newEvent, getType());
		if (newPanel != null) {
			events.add(newPanel);
			newPanel.addParentDateEventHandler(this);
			repaintData();
		}
	}

	public void removeEventData(T newEvent) {
		for (EventPanel<? extends T> ev : events) {
			if (renderer.equal(ev.getValue(), newEvent)) {
				events.remove(ev);
				ev.clearParent();
				repaintData();
				return;
			}
		}
	}

	public void clearData() {
		for (EventPanel<? extends T> panelToRemove : events) {
			panelToRemove.clearParent();
		}
		events.clear();
		repaintData();
	}

	@SuppressWarnings("unchecked")
	public void addEventsByList(List<? extends T> newEvents) {
		events.clear();
		
		if (newEvents != null) {
			for (T newEvent : newEvents) {
				EventPanel<T> newPanel = (EventPanel<T>) renderer.createPanel(newEvent, getType());
				if (newPanel != null) {
					events.add(newPanel);
					newPanel.addParentDateEventHandler(this);
				}
			}
		}
		repaintData();
	}

	private HashMap<String, DateAnchor> dates = new HashMap<String, DateAnchor>();

	private void repaintData() {
		listPanel.clear();
		DOM.setStyleAttribute(listPanel.getElement(), "width", "100%");
		// DOM.setStyleAttribute(monthPanel.getElement(), "height", "100%");

		listPanel.resizeRows(events.size());
		Collections.sort(events, new EventComparator<T>());
		final HashSet<String> date = new HashSet<String>();
		int days = 0;
		int row = 0;
		for (EventPanel<? extends T> eventPanel : events) {
			String dateEvent = renderer.getDay(eventPanel.getStart());
			if (date.contains(dateEvent)) {
				listPanel.setWidget(row, 0, new HTML("&nbsp;"));
			} else {
				days++;
				DateAnchor ref = dates.get(dateEvent);
				if (ref == null) {
					ref = new DateAnchor(dateEvent, eventPanel.getStart());
					ref.addClickHandler(this);
				}
				listPanel.setWidget(row, 0, ref);
				date.add(dateEvent);
			}
			if (renderer.isWholeDayEvent(eventPanel.getValue())) {
				listPanel.setWidget(row, 1, new HTML("&nbsp;"));
			} else {
				if (renderer.getEndTime(eventPanel.getValue()) != null) {
					listPanel.setWidget(row, 1, new Label(renderer.getTime(eventPanel.getStart()) + " - " + renderer.getTime(eventPanel.getEnd())));
				} else {
					listPanel.setWidget(row, 1, new Label(renderer.getTime(eventPanel.getStart())));
				}
			}

			listPanel.getCellFormatter().setVerticalAlignment(row, 0, HorizontalPanel.ALIGN_TOP);
			listPanel.getCellFormatter().setVerticalAlignment(row, 1, HorizontalPanel.ALIGN_TOP);

			listPanel.setWidget(row, 2, eventPanel);
			eventPanel.repaintTime();
			listPanel.getCellFormatter().setWidth(row, 0, "60px");
			listPanel.getCellFormatter().setWidth(row, 1, "110px");
			listPanel.getCellFormatter().setHorizontalAlignment(row, 2, HorizontalPanel.ALIGN_LEFT);
			if (!General.even(days)) {
				setStyleProperty(row, CSS.A.BACKGROUND_COLOR, "#FFFFFF");
			} else {
				setStyleProperty(row, CSS.A.BACKGROUND_COLOR, "#e8eef7");
			}
			setStyleProperty(row, CSS.A.PADDING_TOP, "3px");
			setStyleProperty(row, CSS.A.PADDING_BOTTOM, "3px");
			setStyleProperty(row, CSS.A.PADDING_LEFT, "3px");

			setStyleProperty(row, CSS.A.FONT_WEIGHT, CSS.V.FONT_WEIGHT.BOLD);
			row++;
		}
	}

	private void setStyleProperty(int row, String name, String value) {
		setStyleProperty(row, 0, name, value);
		setStyleProperty(row, 1, name, value);
		setStyleProperty(row, 2, name, value);
	}

	private void setStyleProperty(int row, int column, String name, String value) {
		CSS.setProperty(listPanel.getCellFormatter().getElement(row, column), name, value);
	}

	public void onResize(ResizeEvent event) {

	}

	public void notifyReady(DateEventActions action, T data, boolean reload) {
		if (reload) {
			loadData();
		}
		switch (action) {
		case ADD_DONE: {
			addEventData(data);
			break;
		}
		case REMOVE_DONE: {
			removeEventData(data);
			break;
		}
		case UPDATE_DONE: {
			updateEventData(data);
			break;
		}
		default:
			break;
		}
		DateEvent.fire(this, action, data);
	}

	public void loadData() {
		clearData();
		renderer.getEventsForRange(getFirstDateShow(), getLastDateShow(), this, true);
	}

	public void handleDateEvent(DateEvent<T> newEvent) {
		switch (newEvent.getCommand()) {
		case DRAG_DROP: {
			renderer.updateEvent(renderer.createUpdateHandler(this, newEvent.getData()));
			break;
		}
		case UPDATE: {
			renderer.updateEvent(renderer.createUpdateHandler(this, newEvent.getData()));
			break;
		}
		case REMOVE: {
			renderer.removeEvent(renderer.createRemoveHandler(this, newEvent.getData()));
			break;
		}
		case ADD: {
			renderer.addEvent(renderer.createAddHandler(this, newEvent.getData()));
			break;
		}
		case EDIT: {
			renderer.editAfterClick(newEvent.getData(), this);
			break;
		}
		case ADD_DONE: {
			addEventData(newEvent.getData());
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		case REMOVE_DONE: {
			removeEventData(newEvent.getData());
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		case UPDATE_DONE: {
			updateEventData(newEvent.getData());
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		case RELOAD: {
			loadData();
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
		default:
			DateEvent.fire(this, newEvent.getDate(), newEvent.getCommand(), newEvent.getData());
			break;
		}
	}

	public HandlerRegistration addDateEventHandler(DateEventListener<? extends T> handler) {
		return addHandler(handler, DateEvent.getType());
	}

	public DateRenderer<T> getRenderer() {
		return renderer;
	}

	final Calendar current = new GregorianCalendar();

	public Date next() {
		current.add(Calendar.WEEK_OF_YEAR, 1);
		setLast();
		return current.getTime();
	}

	public Date prev() {
		current.add(Calendar.WEEK_OF_YEAR, -1);
		setLast();
		return current.getTime();
	}

	public Date nextMore() {
		current.add(Calendar.YEAR, 1);
		setLast();
		return current.getTime();
	}

	public Date prevMore() {
		current.add(Calendar.YEAR, -1);
		setLast();
		return current.getTime();
	}

	public void updateEventData(T newEvent) {
		removeEventData(newEvent);
		addEventData(newEvent);
	}

	public Date getFirstDateShow() {
		return current.getTime();
	}

	public Date getLastDateShow() {
		return lastShow;
	}

	public Date getFirstDateLogical() {
		return current.getTime();
	}

	public Date getLastDateLogical() {
		return lastShow;
	}

	private int width = 300;

	public void setPixelSize(int newWidth, int newHeight) {
		if (newWidth > 30) {
			width = newWidth;
		}
		if (newHeight > 15) {
			scroller.setHeight(newHeight - 14 + "px");
		}
		super.setWidth(width + "px");
	}

	public PanelType getType() {
		return PanelType.LIST;
	}

	public void setEvents(List<? extends T> events) {
		clearData();
		addEventsByList(events);
	}

	public void onClick(ClickEvent event) {
		if (event.getSource() instanceof DateAnchor) {
			DateAnchor real = (DateAnchor) event.getSource();
			DateEvent.fire(this, real.getDate(), DateEventActions.GO_TO_DAY_VIEW);
		}

	}

}
